function imp=xfr_fn_to_imp(f,H,Ts,Tsym)
%%% Create impulse response from transfer function in frequency domain  %%%
%%% Impulse response is interpolated to the sample time required by the
%%% simulator
% input 
%   f - frequency in Hz
%   H - transfer function
%   Ts - simulator sample time
%   Tsym - simulator symbol period
% output
%   imp_response

% clear all;
% load hdata;
% Ts=1e-12;
% Tsym=80e-12;
num_fft_pts=2^12;

% set the symbol frequency
f_sym=1/Tsym;
% get the maximum sampling frequency from the transfer function
f_sym_max=2*max(f);
% stop the simulation if the symbol frequency is smaller than the maximum
% measured sampling frequency
if (f_sym > f_sym_max), 
   error('Max input frequency too low for requested symbol rate, can''t interpolate!');
end	

f_sym_max=f_sym*floor(f_sym_max/f_sym);
Hm=abs(H);
Hp=angle(H);

%%% need to force phase to zero at zero frequency to avoid funky behavior
if f(1)==0,
   Hm_ds=[fliplr(Hm(2:end-1)) Hm];
   Hp_ds=[fliplr(-Hp(2:end-1)) Hp];
   fds=[-fliplr(f(2:end-1)) f];
   fds_m = fds; 
   fds_p = fds;
else
   Hm_ds=[fliplr(Hm(1:end-1)) Hm];
   Hp_ds=[fliplr(-Hp(1:end-1)) 0 Hp];
   fds_m=[-fliplr(f(1:end-1)) f];
   fds_p=[-fliplr(f(1:end-1)) 0 f];
end

df=f_sym_max/2/num_fft_pts;
f_ds_interp=-f_sym_max/2+df:df:f_sym_max/2;
Hm_ds_interp=spline(fds_m,Hm_ds,f_ds_interp);
Hp_ds_interp=spline(fds_p,unwrap(Hp_ds),f_ds_interp);

Hm_ds_interp_sh=fftshift(Hm_ds_interp);
Hp_ds_interp_sh=fftshift(Hp_ds_interp);

H_ds_interp_sh=Hm_ds_interp_sh.*exp(j*Hp_ds_interp_sh);

% impulse response from ifft of interpolated frequency response

imp=ifft(H_ds_interp_sh);
imp_r=real(imp);

dt_sym=1/f_sym_max;

%refit data into simulator's time step
dt_time=0:dt_sym:dt_sym*(length(imp_r)-1);
time = 0:Ts:dt_time(end); 
imp = interp1(dt_time, imp_r, time, 'spline')*Ts/dt_sym;

% figure(1000)
% plot(imp);

return
   

